const express = require('express');
const multer = require('multer');
const claim_details = express();
// Middleware for parsing JSON
claim_details.use(express.json());
const connection = require('../config');

const upload = multer({ storage:multer.memoryStorage() }); //Define a directory to store uploaded files

//Get method
claim_details.get('/claim_details', (req, res) => {
  const query = 'SELECT * FROM claim_details';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve coupon' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No coupon found' });
    } else {
      res.json(results);
    }
  });
});
//get by claimed_by id
// Get method for retrieving a claim by claimed_by
claim_details.get('/get-claim_details/:claimed_by', (req, res) => {
  // Extract the claimed_by parameter from the URL
  const claimedBy = req.params.claimed_by;

  // SQL query to select claims with the specified claimed_by
  const query = 'SELECT * FROM claim_details WHERE claimed_by = ?';

  connection.query(query, [claimedBy], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve claim' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No claim found for claimed_by: ' + claimedBy });
    } else {
      res.json(results);
    }
  });
});

//get by id
claim_details.get('/claim_details/:id', (req, res) => {   //retreiving all the shop names where created_by
  const id = req.params.id;

  // Execute the SQL query to retrieve shop names for the provided created_by user ID
  const query = 'SELECT * FROM claim_details WHERE id = ?';
  connection.query(query, [id], (err, result) => {
    if (err) {
      console.error('Error retrieving Mysql query:', err);
      return res.status(500).json({ error: 'Internal server error' });
    }
    return res.json(result);
  });
});

// Post method
claim_details.post('/upload', upload.single('claim_doc'), (req, res) => {

    const {donor_name,claimed_by, donor_mobile, donor_blood_group,patient_name,blood_donated_date,raiser_name,raiser_mobile,raiser_email,is_verify } = req.body;
    const imageFilePath = req.file.buffer.toString('base64'); // This is the path to the uploaded image file
  
    // Insert file information into the MySQL database
    const query = 'INSERT INTO claim_details (claim_doc,claimed_by,donor_name, donor_mobile, donor_blood_group,patient_name,blood_donated_date,raiser_name,raiser_mobile,raiser_email,is_verify, created_at, updated_at) VALUES (?,?,?, ?, ?, ?,?,?, ?, ?, ?, NOW(), NOW())';
    connection.query(query, [imageFilePath,claimed_by,donor_name, donor_mobile, donor_blood_group,patient_name,blood_donated_date,raiser_name,raiser_mobile,raiser_email,is_verify], (err, result) => {
      if (err) {
        console.error('Error inserting data into the database: ', err);
        return res.status(500).json({ error: 'Internal Server Error' });
      }
      
      // Get the ID of the newly inserted record
      const insertedId = result.insertId;
  
      // Create a response object with all the data, including the ID
      const responseData = {
        id: insertedId,
        claim_doc: imageFilePath,
        claimed_by,
        donor_name,
         donor_mobile, 
         donor_blood_group,
         patient_name,
         blood_donated_date,
         raiser_name,
         raiser_mobile,
         raiser_email,
         is_verify,
        created_at: new Date(), // You can use a proper date format here
        updated_at: new Date(), // You can use a proper date format here
      };
  
      res.status(200).json(responseData); // 200 status code indicates resource creation
  
    });
  });
  
// PUT method for updating a record
claim_details.put('/update/:id', upload.single('claim_doc'), (req, res) => {
  const claimId = req.params.id; // Get the ID of the claim to be updated

  const {
    donor_name,
    claimed_by,
    donor_mobile,
    donor_blood_group,
    patient_name,
    blood_donated_date,
    raiser_name,
    raiser_mobile,
    raiser_email,
    is_verify,
  } = req.body;

  let imageFilePath;
  if (req.file) {
    // If a new image is uploaded, update the imageFilePath
    imageFilePath = req.file.buffer.toString('base64');
  }

  // Construct the SQL UPDATE query
  const updateQuery =
    'UPDATE claim_details SET ' +
    'claim_doc = COALESCE(?, claim_doc), ' +
    'claimed_by = ?, ' +
    'donor_name = ?, ' +
    'donor_mobile = ?, ' +
    'donor_blood_group = ?, ' +
    'patient_name = ?, ' +
    'blood_donated_date = ?, ' +
    'raiser_name = ?, ' +
    'raiser_mobile = ?, ' +
    'raiser_email = ?, ' +
    'is_verify = ?, ' +
    'updated_at = NOW() ' +
    'WHERE id = ?';

  const queryParams = [
    imageFilePath,
    claimed_by,
    donor_name,
    donor_mobile,
    donor_blood_group,
    patient_name,
    blood_donated_date,
    raiser_name,
    raiser_mobile,
    raiser_email,
    is_verify,
    claimId,
  ];

  connection.query(updateQuery, queryParams, (err, result) => {
    if (err) {
      console.error('Error updating data in the database: ', err);
      return res.status(500).json({ error: 'Internal Server Error' });
    }

    // Check if a record was updated
    if (result.affectedRows === 0) {
      return res.status(404).json({ error: 'Claim not found' });
    }

    // Create a response object with all the data
    const responseData = {
      id: claimId,
      claim_doc: imageFilePath || 'Image not updated', // Provide appropriate message if image not updated
      claimed_by,
      donor_name,
      donor_mobile,
      donor_blood_group,
      patient_name,
      blood_donated_date,
      raiser_name,
      raiser_mobile,
      raiser_email,
      is_verify,
      updated_at: new Date(), // You can use a proper date format here
    };

    res.status(200).json(responseData); // 200 status code indicates success
  });
});

// DELETE method for deleting a coupon by ID
claim_details.delete('/delete/:id', (req, res) => {
    const couponId = req.params.id; // Get the ID from the URL parameter

    // Delete the coupon from the MySQL database
    const deleteQuery = 'DELETE FROM claim_details WHERE id = ?';

    connection.query(deleteQuery, [couponId], (err, result) => {
        if (err) {
            console.error('Error deleting data from the database: ', err);
            return res.status(500).json({ error: 'Internal Server Error' });
        }

        if (result.affectedRows === 0) {
            // No rows were deleted (coupon with given ID not found)
            return res.status(404).json({ error: 'Coupon not found' });
        }

        res.status(200).json({ message: 'Coupon successfully deleted', deletedId: couponId });
    });
});


module.exports = claim_details;